                               %OBJECTIVES

%Provides the constants for the CO2 flux model and for the Decomposition of
%the Differential in the CO2 Fluxes. The latter needs the coefficients of
%the polinomial derived from Newton's formula for the numerical estimation
%of the partial derivatives

%--------------------------------------------------------------------------
                               %CONSTANTS


% A and B constants for roughness length (z0) estimated from wave height
%(Hs) and wave length (Lp) according to Taylor and Yelland (2001)
Aw=1.26;
Bw=1.2;

% Wave length (Lp) calibration for buoy data
Omega=0.355;

% Specific Heat of air
cp=1.005;                                                          %J/(g*K)

% gravitational acceleration
g=9.80665;                                                           %m/s^2
                               
% Henry's law constant for CO2 in pure water at 25Celsius. K=p/c form.
kHpc=29.4118;                                                   % L.atm/mol

% Mass fraction (MF) of solutes in standard sea water as a proportion of
% total salinity.
MassFraction= ...
{'Compound' 'MF'  'v1'     'v2'      'v3'     'v4'       'v5'      'v6'   ;
  'NaCl'    0.798  16.22    1.3229    1.4849   0.0074691  30.78    2.0583 ;
  'KCl'     0.022   6.4883  1.3175   -0.7785   0.09272    -1.3     2.0811 ;
  'CaCl2'   0.033  32.028   0.787792 -1.1495   0.0026995  780860   5.8442 ;
  'MgCl2'   0.047  24.032   2.2694    3.7108   0.021853   -1.1236  0.14474;
  'MgSO4'   0.100  72.269   2.2238    6.6037   0.0079004  3340.1   6.1304};

% Relative molecular mass of air
Ma=28.97;

% Relative molecular mass of CO2
Mb=44.01;

% Relative molecular mass of H20
Ms=18.01;

% Association factor of H20
phi=2.6;

% Universal ideal gas law constant
R=8.314472;

% Rho A parameters
Sd0=1.293393662;
Sd1=-5.538444326*10^-3;
Sd2=3.860201577*10^-5;
Sd3=-5.2536065*10^-7;

% Eta A parameters
Sv0=1.715747771*10^-5;
Sv1=4.722402075*10^-8;
Sv2=-3.663027156*10^-10;
Sv3=1.873236686*10^-12;
Sv4=-8.050218737*10^-14;

% Rhow0 (pure water) parameters
r0=999.842594;
r1=0.06793952;
r2=-0.00909529;
r3=0.0001001685;
r4=-0.000001120083;
r5=0.000000006536332;

% Rhow (marine water) parameters
a0=0.824493;
a1=-0.0040899;
a2=0.000076438;
a3=-0.00000082467;
a4=0.0000000053875;
b0=-0.00572466;
b1=0.00010277;
b2=-0.0000016546;
c=0.00048314;

% Temperature dependence of the solution: -delta_soln*H/R
tds=2400;

% Liquid molar volume of air 
Va=20.1;                                                          %cm^3/mol

% Liquid molar volume of CO2 at its boiling point estimated 
% using the additive Schroeder method: molecular structure of CO2 is O=C=O,
% increments of C=7, O=7 and double bound=7, therefore:
Vb=1*7+2*7+2*7;                                                   %cm^3/mol 

% von Krmn's constant for friction velocity estimates
vK=0.4;

