                                %OBJECTIVES:

%To estimate the C02 air-water flux.

%--------------------------------------------------------------------------

                            %CALCULUS: Flux 

% Flux estimate from Fick's Law
switch FluxNumScheme
    
    case 'SingleLayer'
        % Air to water flux adapted from Goddard (2006), Zappa et al (2007)
        % Similar to Fw by Johnson 2010
        if fluxunit==1
            Fw=Kw.*(Cg_./kH-Cl_);                              %mol/(m^2*s)
        elseif fluxunit==2
            Fw=Kw.*(Cg_./kH-Cl_)*1000*60*60;                  %mmol/(m^2*h)
        elseif fluxunit==3
            Fw=Kw.*(Cg_./kH-Cl_)*1000*60*60*24;             %mmol/(m^2*day)
        end
        
    case 'DoubleLayer'     
        % Air to water flux adapted from Liss and Slater (1974)
        
        % Total transfer velocity from the air point of view 
        Ka=(1./ka+kH./kw).^-1;                                        % m/s

        % Total transfer velocity from the water point of view 
        Kw=(1./kw+1./(kH.*ka)).^-1;                                   % m/s

        % Flux (air to the water) estimated from the air point of view
        if fluxunit==1
            Fa=Ka.*(Cg_-kH.*Cl_);                              %mol/(m^2*s)
        elseif fluxunit==2
            Fa=Ka.*(Cg_-kH.*Cl_)*1000*60*60;                  %mmol/(m^2*h)
        elseif fluxunit==3    
            Fa=Ka.*(Cg_-kH.*Cl_)*1000*60*60*24;             %mmol/(m^2*day)
        end

        % Flux (air to the water) estimated from the water point of view
        if fluxunit==1
            Fw=Kw.*(Cg_./kH-Cl_);                              %mol/(m^2*s)
        elseif fluxunit==2
            Fw=Kw.*(Cg_./kH-Cl_)*1000*60*60;                  %mmol/(m^2*h)
        elseif fluxunit==3
            Fw=Kw.*(Cg_./kH-Cl_)*1000*60*60*24;             %mmol/(m^2*day)
        end
           
end
