                                 %OBJECTIVES

%To estimate the C02 air-water flux.

%To test different numerical schemes used to estimate the flux.

%To test different equations used to estimate the influence of such factors
%as air and water temperature, air presure, salinity, air velocity, 
%water velocity and depth.

%--------------------------------------------------------------------------
                                %CALCULUS

for t=1:size(Ta,2)
    
    Hs_=Hs(t);
    MO_L_=MO_L(t);
    Lp_=Lp(t);
    P_=P(t);
    S_=S(t);
    Ta_=Ta(t);
    Tw_=Tw(t);
    u10_=u10(t);
    vhf_=vhf(t);
    w_=w(t);
    Wp_=Wp(t);
    z_=z(t);
    z0_=z0(t)/1000;
    AS_alpha_=AS_alpha(t);
    

    %Conversion of CO2 air concentration from partial pressure (ppm) to
    %mol/m^3 if originally given in ppm.
    if isnan(Cg)==0
        Cg_=Cg(t);
    elseif isnan(Cg)==1
        Tka=273.15+Ta_;                                             %Kelvin
        Pascal=P_*101325.01;                                            %Pa
        Cg_=Cgppm(t)*Pascal/(10^6*R*Tka);                          %mol/m^3
    end

    % Conversion of CO2 water concentration from equivalent atmospheric 
    %partial pressure in equilibrium (ppm) to mol/m^3 if originally given 
    %in ppm.
    if isnan(Cl)==0
        Cl_=Cl(t);
    elseif isnan(Cl)==1
        Tka=273.15+Ta_;
        Tkw=273.15+Tw_;
        run kHExe
        kHcp=12.1866/(Tka*kH);                                 %mol/(L*atm)
        Cl_=Clppm(t)*kHcp*P_*10^-3;                               %mol/m^3
    end
    
    % Estimates air-side transfer velocity
    T=Ta_;
    Tka=273.15+Ta_;                            %Celsius to Kelvin conversor
    run kaExe

    %Estimates Henry's constant and water-side transfer velocity
    T=Tw_;
    Tkw=273.15+Tw_;                            %Celsius to Kelvin conversor
    run kHExe
    run kwExe

    % Estimates the flux
    run FluxExe

%--------------------------------------------------------------------------

                             % SAVE and CLEAN

    Parameters(t).air.temperature=Ta_;
    Parameters(t).air.pressure=P_;
    Parameters(t).air.wind=u10_;
    Parameters(t).air.CO2=Cg_;
    Parameters(t).water.temperature=Tw_;
    Parameters(t).water.salinity=S_;
    Parameters(t).water.current=w_;
    Parameters(t).water.depth=z_;
    Parameters(t).water.CO2=Cl_;

    NumScheme.flux=FluxNumScheme;
    NumScheme.flux_units=fluxunit;
    NumScheme.Ka=KaScheme;
    NumScheme.CD=DragCoef;
    NumScheme.Kw=KwScheme;
    NumScheme.SchmidtNumber=SchmidtScheme;
    NumScheme.Dw=DwScheme;

    Results(t).Henry_constant=kH;
    Results(t).transfer_velocity.friction_velocity=u_;
    Results(t).transfer_velocity.roughness_length=z0_;
    Results(t).transfer_velocity.water.wind=kwWind;
    Results(t).transfer_velocity.water.current=kwCurrent;
    Results(t).transfer_velocity.water.alfa=Alfa;
    Results(t).transfer_velocity.water.kw=kw;
    Results(t).Flux.water_based=Fw;
    Results(t).chemicalCO2depletion=Cg_/kH-Cl_;

    try
        Results(t).transfer_velocity.water.Dw=Dw;
    catch
        'No water diffusivity scheme'
    end

    try
        Results(t).transfer_velocity.air=ka;
        Results(t).transfer_velocity.total_air_based=Ka;
        Results(t).transfer_velocity.total_water_based=Kw;
        Results(t).Flux.air_based=Fa;
    catch
        'SingleLayer flux scheme: no ka, Ka, Kw or Fa'
    end
end

clear('-regexp','^A|^C|^Dw|^E|^F|^H|^K|^L|^M|^Rb|^Rh|^S|^T|^U|^V|^W|^a|^b')
clear('-regexp','^d|^k|^p|^r|^t|^u|^v|^w|^z')
clear Bw c cp Da DragCoef delta epsilon fTS fluxunit g h i msi msj n Omega  
clear P P_ Pascal R y_at_k
