                               %OBJECTIVES

%Executes the decomposition of the differential between fluxes (DDF) in its
%several components

%--------------------------------------------------------------------------

                                %CALCULUS

nvar=size(n,2);

%Conversion of CO2 air concentration from partial pressure (ppm) to mol/m^3
%or to mmol/m^3 if originally given in ppm.
if isnan(Cgppm)==0
    Tk=273.15+Ta;                                                   %Kelvin
    Pascal=P*101325.01;                                                 %Pa
    Cg=Cgppm.*Pascal./(10^6*R*Tk);                                 %mol/m^3    
end
Cg_m=Cg.*1000;                                                    %mmol/m^3

%Conversion of air pressure units from atm to KiloPascal.
KPa=101.32501.*P;                                                      %KPa

%Conversion of wind speed units.
if u10_units==1
    u10_ddf=u10;                                                       %m/s
elseif u10_units==2
    u10_ddf=u10.*60.*60./1000;                                        %km/h
end

%Conversion of roughness length units.
if z0_units==1
    z0_ddf=z0.*10;                                                   %mm/10
elseif z0_units==2
    z0_ddf=z0;                                                          %mm
elseif z0_units==3
    z0_ddf=z0./10;                                                      %cm
elseif z0_units==4
    z0_ddf=z0./100;                                                     %dm
end

%Conversion of Atmospheric Stability alpha units.
if AS_alpha_units==1
    AS_alpha_ddf=AS_alpha;                                          %scalar
elseif AS_alpha_units==2
    AS_alpha_ddf=AS_alpha.*10;                                      %scalar
end

%Conversion of Monin-Obukhov length L units.
if MO_L_units==1
    MO_L_ddf=MO_L;                                                       %m
elseif MO_L_units==2
    MO_L_ddf=MO_L./10;                                                 %dam
elseif MO_L_units==3
    MO_L_ddf=MO_L./100;                                                 %hm
end

% Conversion of CO2 water concentration from equivalent atmospheric partial
% pressure in equilibrium (ppm) to mol/m^3 or to mmol/m^3 if originally 
% given in ppm.
if isnan(Clppm)==0
    S_=S;
    Tka=273.15+Ta;
    Tkw=273.15+Tw;
    run kHExe
    kHcp=12.1866./(Tka.*kH);                                   %mol/(L*atm)
    Cl=Clppm.*kHcp.*P*10^-3;                                       %mol/m^3
end
Cl_m=Cl.*1000;                                                    %mmol/m^3

clear kHt Pascal S_
clear -regexp '^kHc|^Tk'

%Conversion of current speed units.
if w_units==1
    w_ddf=w;                                                          %dm/s
elseif w_units==2
    w_ddf=w.*100;                                                     %cm/s
elseif w_units==3
    w_ddf=w.*10;                                                      %dm/s    
elseif w_units==4
    w_ddf=w.*60;                                                     %m/min
elseif w_units==5
    w_ddf=w.*60.*60./100;                                             %hm/h
elseif w_units==6
    w_ddf=w.*60.*60./1000;                                            %km/h
end

%Conversion of depth units.
if z_units==1
    z_ddf=z.*10;                                                        %dm
elseif z_units==2
    z_ddf=z;                                                             %m
elseif z_units==3
    z_ddf=z./10;                                                       %dam
elseif z_units==4
    z_ddf=z./100;                                                       %hm
end

%Conversion of significant wave height units.
if Hs_units==1
    Hs_ddf=Hs*100;                                                      %cm
elseif Hs_units==2
    Hs_ddf=Hs*10;                                                       %dm
elseif Hs_units==3
    Hs_ddf=Hs;                                                           %m
end

%Conversion of peak wave length units.
if Lp_units==1
   Lp_ddf=Lp;                                                            %m
elseif Lp_units==2
    Lp_ddf=Lp./10;                                                     %dam
elseif Lp_units==3
    Lp_ddf=Lp./100;                                                     %hm
end

%Conversion of peak wave period units.
if Wp_units==1
   Wp_ddf=Wp;                                                            %s
elseif Wp_units==2
    Wp_ddf=Wp./10;                                                     %10s                                                      %hm
end

%Decomposition of the difference in CO2 fluxes.
for site=1:size(Ta,1)-1
site %to trace calculus evolution in the command prompt

    for t=1:size(Ta,2)

        %Xa and F(a)
        Cg_=Cg(1,t);
        Ta_=Ta(1,t);
        P_=P(1,t);
        u10_=u10(1,t);
        Cl_=Cl(1,t);
        Tw_=Tw(1,t);
        S_=S(1,t);
        w_=w(1,t);
        z_=z(1,t);
        z0_=z0(1,t)/1000;
        Wp_=Wp(1,t);
        Hs_=Hs(1,t);
        Lp_=Lp(1,t);
        vhf_=vhf(1,t);
        MO_L_=MO_L(1,t);
        AS_alpha_=AS_alpha(1,t);
        T=Ta_;
        Tka=273.15+Ta_;
        run kaExe
        T=Tw_;
        Tkw=273.15+Tw_;
        run kHExe
        run kwExe
        run FluxExe
        Fxa(t)=Fw;
        
        %Xb and F(b)
        Cg_=Cg(1+site,t);
        Ta_=Ta(1+site,t);
        P_=P(1+site,t);
        u10_=u10(1+site,t);
        Cl_=Cl(1+site,t);
        Tw_=Tw(1+site,t);
        S_=S(1+site,t);
        w_=w(1+site,t);
        z_=z(1+site,t);
        z0_=z0(1+site,t)/1000;
        Wp_=Wp(1+site,t);
        Hs_=Hs(1+site,t);
        Lp_=Lp(1+site,t);
        vhf_=vhf(1+site,t);
        MO_L_=MO_L(1+site,t);
        AS_alpha_=AS_alpha(1+site,t);
        T=Ta_;
        Tka=273.15+Ta_;
        run kaExe
        T=Tw_;
        Tkw=273.15+Tw_;
        run kHExe
        run kwExe
        run FluxExe
        Fxb(t,site)=Fw;
        
        %difference in the environmental variables
        h=[Cg_m(1+site,t)-Cg_m(1,t);
           Ta(1+site,t)-Ta(1,t);
           KPa(1+site,t)-KPa(1,t);
           u10_ddf(1+site,t)-u10_ddf(1,t);
           z0_ddf(1+site,t)-z0_ddf(1,t);
           MO_L_ddf(1+site,t)-MO_L_ddf(1,t);
           AS_alpha_ddf(1+site,t)-AS_alpha_ddf(1,t);
           Cl_m(1+site,t)-Cl_m(1,t);
           Tw(1+site,t)-Tw(1,t);
           S(1+site,t)-S(1,t);
           w_ddf(1+site,t)-w_ddf(1,t);
           z_ddf(1+site,t)-z_ddf(1,t);
           Hs_ddf(1+site,t)-Hs_ddf(1,t);
           Lp_ddf(1+site,t)-Lp_ddf(1,t);
           Wp_ddf(1+site,t)-Wp_ddf(1,t);
           vhf(1+site,t)-vhf(1,t)];
       
        hi=find(isnan(h)==1);
        h(hi)=0;
            
        if isnan(Cgppm)==0
            h(1)=Cgppm(1+site,t)-Cgppm(1,t);
        end
       
        if isnan(Clppm)==0
            h(8)=Clppm(1+site,t)-Clppm(1,t);
        end
        
        %Estimation of loose delta(i)
        delta_a=delta';
        loose=find(isnan(delta)==1);
        delta_a(loose)=abs(h(loose)'./n(loose));
       
        %Direction of the estimation of the partial derivatives (+ or -)
        delta_b=h./abs(h).*delta_a;
        i=find(h==0);
        delta_b(i)=1;
        i=find(isnan(h)==1);
        delta_b(i)=0;
        aux=find(delta_b>=0);
        forward_j=zeros(nvar,1);
        forward_j(aux)=1;
        aux=find(delta_b<0);
        backward_j=zeros(nvar,1);
        backward_j(aux)=1;
        
        clear i T
        clear -regexp _$|^Tk
        
        %Array with CO2 flux estimates
        ti=cputime;
        N=n+ones(size(n));
        y=zeros(N);
        n_mp=prod(N);
        
        for ji=1:n_mp
            [i1,i2,i3,i4,i5,i6,i7,i8,i9,i10,i11,i12,i13,i14,i15,i16]=...
                ind2sub(N,ji);
           
            Cg_=(Cg_m(1,t)+(i1-1)*delta_b(1))/1000;
            Ta_=Ta(1,t)+(i2-1)*delta_b(2);
            Tka=273.15+Ta_;
            P_=(KPa(1,t)+(i3-1)*delta_b(3))/101.32501;
            
            if u10_units==1
                u10_=(u10_ddf(1,t)+(i4-1)*delta_b(4));
            elseif u10_units==2
                u10_=(u10_ddf(1,t)+(i4-1)*delta_b(4))*1000/3600;
            end
            
            if z0_units==1
                z0_=(z0_ddf(1,t)+(i5-1)*delta_b(5))/10000;
            elseif z0_units==2
                z0_=(z0_ddf(1,t)+(i5-1)*delta_b(5))/1000;
            elseif z0_units==3
                z0_=(z0_ddf(1,t)+(i5-1)*delta_b(5))/100;
            elseif z0_units==4
                z0_=(z0_ddf(1,t)+(i5-1)*delta_b(5))/10;
            end
            
            if MO_L_units==1
                MO_L_=(MO_L_ddf(1,t)+(i6-1)*delta_b(6));
            elseif MO_L_units==2
                MO_L_=(MO_L_ddf(1,t)+(i6-1)*delta_b(6))*10;
            elseif MO_L_units==3
                MO_L_=(MO_L_ddf(1,t)+(i6-1)*delta_b(6))*100;
            end
            
            if AS_alpha_units==1
                AS_alpha_=(AS_alpha_ddf(1,t)+(i7-1)*delta_b(7));
            elseif MO_L_units==2
                AS_alpha_=(AS_alpha_ddf(1,t)+(i7-1)*delta_b(7))/10;
            end
            
            Cl_=(Cl_m(1,t)+(i8-1)*delta_b(8))/1000;
            Tw_=Tw(1,t)+(i9-1)*delta_b(9);
            Tkw=273.15+Tw_;
            S_=S(1,t)+(i10-1)*delta_b(10);
            
            if w_units==1
                w_=w_ddf(1,t)+(i11-1)*delta_b(11);
            elseif w_units==2
                w_=(w_ddf(1,t)+(i11-1)*delta_b(11))/100;
            elseif w_units==3
                w_=(w_ddf(1,t)+(i11-1)*delta_b(11))/10;
            elseif w_units==4
                w_=(w_ddf(1,t)+(i11-1)*delta_b(11))/60;
            elseif w_units==5
                w_=(w_ddf(1,t)+(i11-1)*delta_b(11))*100/3600;
            elseif w_units==6
                w_=(w_ddf(1,t)+(i11-1)*delta_b(11))*1000/3600;
            end
            
            if z_units==1
                z_=(z_ddf(1,t)+(i12-1)*delta_b(12))/10;
            elseif z_units==2
                z_=(z_ddf(1,t)+(i12-1)*delta_b(12));
            elseif z_units==3
                z_=(z_ddf(1,t)+(i12-1)*delta_b(12))*10;
            elseif z_units==4
                z_=(z_ddf(1,t)+(i12-1)*delta_b(12))*100;
            end
            
            if Hs_units==1
                Hs_=(Hs_ddf(1,t)+(i13-1)*delta_b(13))/100;
            elseif Hs_units==2
                Hs_=(Hs_ddf(1,t)+(i13-1)*delta_b(13))/10;
            elseif Hs_units==3
                Hs_=(Hs_ddf(1,t)+(i13-1)*delta_b(13));
            end
            
            if Lp_units==1
                Lp_=(Lp_ddf(1,t)+(i14-1)*delta_b(14));
            elseif Lp_units==2
                Lp_=(Lp_ddf(1,t)+(i14-1)*delta_b(14))*10;
            elseif Lp_units==3
                Lp_=(Lp_ddf(1,t)+(i14-1)*delta_b(14))*100;
            end
            
            if Wp_units==1
                Wp_=(Wp_ddf(1,t)+(i15-1)*delta_b(15));
            elseif Wp_units==2
                Wp_=(Wp_ddf(1,t)+(i15-1)*delta_b(15))*10;
            elseif Wp_units==3
                Wp_=(Wp_ddf(1,t)+(i15-1)*delta_b(15))*100;
            end
            
            vhf_=vhf(1,t)+(i16-1)*delta_b(16);
           
            if isnan(Cgppm)==0
                Cgppm_=(Cgppm(1,t)+(i1-1)*delta_b(1));
                Pascal=P_*101325.01;
               Cg_=Cgppm_.*Pascal./(10^6*R*Tka);
            end

            if isnan(Clppm)==0
                Clppm_=(Clppm(1,t)+(i8-1)*delta_b(8));
                run kHExe
                kHcp=12.1884./(Tka.*kH);
                Cl_=Clppm_.*kHcp.*P_*10^-3;
            end
            
            T=Ta_;
            run kaExe
            T=Tw_;
            run kHExe
            run kwExe
            run FluxExe
            y(ji)=Fw;
            
        end
        dt1=cputime-ti
        clear dt1 Fw i ji
        clear -regexp '_$'
        
        
%1. To estimate the Delta array
        ti=cputime;
        hyperD=ones(n_mp,1);
        
        for i1=1:n_mp

    [m1,m2,m3,m4,m5,m6,m7,m8,m9,m10,m11,m12,m13,m14,m15,m16]=ind2sub(N,i1);
            m=[m1;m2;m3;m4;m5;m6;m7;m8;m9;m10;m11;m12;m13;m14;m15;m16];
            msum=sum(m);
            mprod=prod(m);
            
            d=zeros(1,mprod);
            
            ii=1:mprod;
            [j1,j2,j3,j4,j5,j6,j7,j8,j9,j10,j11,j12,j13,j14,j15,j16]=...
                                                            ind2sub(m',ii);
            j=[j1;j2;j3;j4;j5;j6;j7;j8;j9;j10;j11;j12;j13;j14;j15;j16];

%1.1 To estimate each Delta (each hyperD: it is the sum of each d array)
            msum=kron(msum,ones(1,mprod));
            jsum=sum(j);
            m=kron(m,ones(1,mprod));
%1.1.1 To estimate Newton's polynomial
         polyNewton=factorial(m-1)./(factorial(j-1).*factorial(m-1-(j-1)));
%1 1.2 To set formula for forward or backward estimation
            fo=kron(forward_j,ones(1,mprod));
            ba=kron(backward_j,ones(1,mprod));
            j_=j.*fo+m.*ba-j.*ba+ba;
%1.1.3 To convert the y indexes from 16D to 1D
            ii=sub2ind(N,j_(1,:),j_(2,:),j_(3,:),j_(4,:),j_(5,:),...
             j_(6,:),j_(7,:),j_(8,:),j_(9,:),j_(10,:),j_(11,:),j_(12,:),...
             j_(13,:),j_(14,:),j_(15,:),j_(16,:));
            d=(-1).^((msum-16)-(jsum-16)).*prod(polyNewton,1).*y(ii);
            hyperD(i1)=sum(d);
        end
        
        hyperD=reshape(hyperD,N);
        dt2=cputime-ti
        clear A ba d d_ fo k polyNewton
        clear -regexp 'J$|^j|^m'

        %Estimating the iK array
        ti=cputime;
        for var=1:nvar
            
            k=dfdx_at_k(var);

            if h(var)>=0   %iK coefficients vector estimated forward

                iK=...
[1 k (k^2-k)/factorial(2) (k^3-3*k^2+2*k)/factorial(3) ...
(k^4-6*k^3+11*k^2-6*k)/factorial(4) (k^5-10*k^4+35*k^3-50*k^2+24*k)/factorial(5);

0 1  (2*k-1)/factorial(2) (3*k^2-6*k+2)/factorial(3) ...
(4*k^3-18*k^2+22*k-6)/factorial(4) (5*k^4-40*k^3+105*k^2-100*k+24)/factorial(5);

0 0 1 (6*k-6)/factorial(3) (12*k^2-36*k+22)/factorial(4) ...
(20*k^3-120*k^2+210*k-100)/factorial(5);

0 0 0 1 (24*k-36)/factorial(4) (60*k^2-240*k+210)/factorial(5);

0 0 0 0 1 (120*k-240)/factorial(5);

0 0 0 0 0 1];

            elseif h(var)<0 %iK coefficients vector estimated backward

                iK=...
[1 -k (k^2-k)/factorial(2) -(k^3-3*k^2+2*k)/factorial(3) ...
(k^4-6*k^3+11*k^2-6*k)/factorial(4) -(k^5-10*k^4+35*k^3-50*k^2+24*k)/factorial(5);

0 -1  (2*k-1)/factorial(2) -(3*k^2-6*k+2)/factorial(3) ...
(4*k^3-18*k^2+22*k-6)/factorial(4) -(5*k^4-40*k^3+105*k^2-100*k+24)/factorial(5);

0 0 1 -(6*k-6)/factorial(3) (12*k^2-36*k+22)/factorial(4) ...
-(20*k^3-120*k^2+210*k-100)/factorial(5);

0 0 0 -1 (24*k-36)/factorial(4) -(60*k^2-240*k+210)/factorial(5);

0 0 0 0 1 -(120*k-240)/factorial(5);

0 0 0 0 0 -1];    
            end

            delta_c=delta_b(var).^(0:theta(var));

            for i_iK=0:theta(var)
                iK_array(var).derivatives(i_iK+1)=...
                             {iK(i_iK+1,1:n(var)+1)./delta_c(i_iK+1)};
            end  
        end

        clear delta_ dfdx_at_k iK i_iK k variable y

        %Finalizing numerical estimates of partial derivatives by doing the
        %element-wise product of the D array with the K array
        Theta=theta+ones(size(theta));
        for j=1:prod(Theta)
            [m1,m2,m3,m4,m5,m6,m7,m8,m9,m10,m11,m12,m13,m14,m15,m16]=...
                ind2sub(Theta,j);

            m(1)=m1; m(2)=m2; m(3)=m3; m(4)=m4; m(5)=m5; m(6)=m6;  m(7)=m7;
            m(8)=m8; m(9)=m9; m(10)=m10; m(11)=m11; m(12)=m12; m(13)=m13;
            m(14)=m14; m(15)=m15; m(16)=m16;

            for var=1:nvar

                if var==1
                    hyperK=...
                    cell2mat(iK_array(var).derivatives(m(var)))';

                elseif var>1
                    auxB=...
                     cell2mat(iK_array(var).derivatives(m(var)));

                    for i_B=1:size(auxB,2)
                        auxC(i_B).field=auxB(i_B).*hyperK;
                    end
                    auxC=auxC(1:i_B);
                    hyperK=cat(var,auxC.field);
                end
            end

        difdxi(m1,m2,m3,m4,m5,m6,m7,m8,m9,m10,m11,m12,m13,m14,m15,m16)=...
        sum(sum(sum(sum(sum(sum(sum(sum(sum(sum(sum(sum(sum(sum(sum(sum(...
                                            hyperD.*hyperK))))))))))))))));
             
    
        end

        clear i_B Theta
        clear -regexp '^aux|^iK|^i\d|^m|^N|^hyper'
        
       %Estimating the coefficients for the terms with h and factorials.
       %This algorithm is 14x faster than using function 'ndgrid' proceeded 
       %by the element-wise multiplication of the 16 provisional arrays
        for var=1:nvar
            if var==1
                hyperH=(h(1).^(0:theta(1))./factorial(0:theta(1)))';

            elseif var>1
                auxB=(h(var).^(0:theta(var))...
                                           ./factorial(0:theta(var)));

                for i_B=1:size(auxB,2)
                    auxC(i_B).field=(auxB(i_B).*hyperH);
                end
                auxC=auxC(1:i_B);
                hyperH=cat(var,auxC.field);
            end
        end

        %Finalizing the Taylor expansion, estimating its sum and the error
        TaylorArray=hyperH.*difdxi;
        TaylorArray(1)=0;
        error1_absolute(t,site)=Fxb(t,site)-Fxa(t)-...
        sum(sum(sum(sum(sum(sum(sum(sum(sum(sum(sum(sum(sum(sum(sum(sum(...
                                               TaylorArray))))))))))))))));
        error1_relative(t,site)=...
                              error1_absolute(t,site)/(Fxb(t,site)-Fxa(t));
        
        clear difdxi i_B theta
        clear -regexp '^aux|^hyper'
        
        %Partition of multivariate terms among variables       
        size(TaylorArray);
        for var=1:nvar
            try
                h2(var)={[0 ones(1,ans(var)-1)]};
            catch
                h2(var)={0};
            end
        end
        
        [H1,H2,H3,H4,H5,H6,H7,H8,H9,H10,H11,H12,H13,H14,H15,H16] = ...
 ndgrid(cell2mat(h2(1)),cell2mat(h2(2)),cell2mat(h2(3)),cell2mat(h2(4)),...
        cell2mat(h2(5)),cell2mat(h2(6)),cell2mat(h2(7)),cell2mat(h2(8)),...
      cell2mat(h2(9)),cell2mat(h2(10)),cell2mat(h2(11)),cell2mat(h2(12)),...
      cell2mat(h2(13)),cell2mat(h2(14)),cell2mat(h2(15)),cell2mat(h2(16)));
              
        sumH=H1+H2+H3+H4+H5+H6+H7+H8+H9+H10+H11+H12+H13+H14+H15+H16;
        i=find(sumH==0);
        sumH(i)=1;
            
        partition(1).field  = H1./sumH;     clear H1
        partition(2).field  = H2./sumH;     clear H2
        partition(3).field  = H3./sumH;     clear H3
        partition(4).field  = H4./sumH;     clear H4
        partition(5).field  = H5./sumH;     clear H5
        partition(6).field  = H6./sumH;     clear H6
        partition(7).field  = H7./sumH;     clear H7
        partition(8).field  = H8./sumH;     clear H8
        partition(9).field  = H9./sumH;     clear H9
        partition(10).field = H10./sumH;    clear H10
        partition(11).field = H11./sumH;    clear H11
        partition(12).field = H12./sumH;    clear H12
        partition(13).field = H13./sumH;    clear H13
        partition(14).field = H14./sumH;    clear H14
        partition(15).field = H15./sumH;    clear H15
        partition(16).field = H16./sumH;    clear H16
        
        clear ans i sumH h2
        
        for var=1:nvar
            DDF(var,t,site)=...
        sum(sum(sum(sum(sum(sum(sum(sum(sum(sum(sum(sum(sum(sum(sum(sum(...
                         partition(var).field.*TaylorArray))))))))))))))));
        end
        
        clear partition var
        
        error2_absolute(t,site)=Fxb(t,site)-Fxa(t)-sum(DDF(:,t,site),1);
        error2_relative(t,site)=...
                              error2_absolute(t,site)/(Fxb(t,site)-Fxa(t));
        dt3=cputime-ti
    end
end
        
clear('-regexp', '^A|^C|^Dw|^Dr|^E|^F|^Hs|^K|^L|^M|^P|^R|^S|^T|^U|^V|^W')
clear('-regexp', '^a|^b|^d|^for|^h|^i|^k|^n|^p|ppm$|^r|site$|^t|^u|^v|^w|^z')
clear Bw c cp Da epsilon fTS fluxunit g Hxy j loose Omega
        
