                              %OBJECTIVES

%Estimates the air side transfer velocity for given temperatures (T),
% air pressures (P) and wind (u10).

%--------------------------------------------------------------------------

                                %CALCULUS:
       %air temperature, pressure and wind on air transfer velocity

                             
% Diffusion coefficient of CO2 in air (cm^2/s) by Fuller et al (1966)
Mr=(Ma+Mb)/(Ma*Mb);
Da=(0.001*Tka.^1.75.*sqrt(Mr))./(P_.*(Va^(1/3)+Vb^(1/3))).^2;

%Dynamic viscosity of air (N.s/m^2)
Etaa=Sv0+Sv1*T+Sv2*T.^2+Sv3*T.^3+Sv4*T.^4;                     

%Air density (kg/m^3)
Rhoa=Sd0+Sd1*T+Sd2*T.^2+Sd3*T.^3;                               

%Kinematic viscosity of air converted from m^2/s to cm^2/s
Upsa=Etaa./Rhoa*10000;

%Schmidt number in air (scalar)
Sca=Upsa./Da;

%Drag coefficient (CD)
switch DragCoef
    case 'Dea91'
        CD=1.3*10^-3;
    case 'M&Y83'
        CD=4*u10_./(10^4);
    case 'Smi80'
        CD=(6.1+0.63.*u10_)./(10^4);
    case 'T&Y01'
        CD=(0.87+0.0752*U10_-0.000661*U10_^2)/10^3;
end

% Friction velocity (m/s)
if isnan(z0_)==1 && isnan(Hs_)==0 && isnan(Lp_)==0
    % roughness length estimated from wave height and wave length
    z0_=Aw*Hs_*(Hs_/(Omega*Lp_))^Bw;                                     %m
end

if isnan(z0_)==1
    %estimated from drag coefficient (CD)
    u_=u10_.*sqrt(CD);
    
elseif isnan(MO_L_)==1 && isnan(vhf_)==0 
    %estimated from Wind Log-Linear Profile with Monin_Obukhof length (L)
    %estimated from vertical heat flux (vhf)
    'error: estimation of MO_L is not yet implemented'
    MO_L_=-u_^3*cp*rhoa*1000*Tpka/(vK*g*vhf);
    psi=AS_alfa.*(10-z0_)./L_;
    u_=u10_.*vK./(log(10)-log(z0_)+psi);
    
elseif isnan(MO_L_)==0 && isnan(vhf_)==1 
    %estimated from Wind Log-Linear Profile with Monin_Obukhof length (L)
    %given as input variable
    psi=AS_alpha_.*(10-z0_)./MO_L_;
    u_=u10_.*vK./(log(10)-log(z0_)+psi);
    
end

% Air transfer velocity (m/s)                                           
switch KaScheme
    
    case 'Joh10'
        if u_~=0
            ka=10^-3+(u10_./u_.^2+5./u_.*Sca.^(2/3)).^-1;
        else
            ka=10^-3;
        end

    case 'Dea91'
        ka=u10_./(770+45*Mb^(1/3));
        
    case 'Lis73'
        ka=0.005+0.21*u10_./100;
        
    case 'M&Y83'
        ka=10^-3+46.2*10^-3*u_.*Sca.^(-2/3);
        
        
    case 'Sea02'
        ka=Da.^0.5*(0.98*u10_+1.26)./100;
        
    case 'J_COA'
        ka=10^-3+u_./(13.3*Sca.^(1/2)+CD.^(-1/2)-5+log(Sca)./0.8);
        
end
